unit DGCompilerThread;

interface

uses
  Windows, Messages, SysUtils, ToolsAPI, Controls, ActiveX, ComObj, Classes;

type
  TDgCustomControl = class(TCustomControl);

  TDGCompiler = class(TThread)
  private
    { Private declarations }
    procedure CompileCurrentProject;
    procedure OutputCompileInfos;
    procedure GetSource;
  protected
    Strs: TStringList;
    CmdReturn, CommandLine, CurrentEditFile, TempFile: string;
    EditBuffer: IOTAEditBuffer;
    procedure Execute; override;
  end;

implementation

uses DGUtils, DGTypes, DGRegister;

{ TDGCompiler }

procedure TDGCompiler.Execute;
begin
  while not Terminated do
  begin
    Sleep(Form.Options.Syntaxs.Interval);
    //  use EditBuffer and EditView, need Synchronize.
    try
      Synchronize(GetSource);
      Sleep(50);
      CompileCurrentProject;
      Sleep(50);
      Synchronize(OutputCompileInfos);
    except
      Debug;
    end;
  end;
end;

procedure TDGCompiler.GetSource;

  function _GetOutputSetting: string;
  var
    WorkDir: string;
  begin
    WorkDir := '"' + Form.Options.Syntaxs.TempDir + '"';
    Result := ' -N' + WorkDir + ' -LE' + WorkDir + ' -LN' + WorkDir + ' -E' + WorkDir;
  end;

  function _GetIncludes: string;
  begin
    Result := ' -I' + Form.Options.Syntaxs.IncludesDir + '"." -R' + Form.Options.Syntaxs.IncludesDir + '"."';
  end;

  function _GetUnits: string;
  begin
    Result := ' -U' + Form.Options.Syntaxs.UnitsDir + '"."';
  end;

begin
  CmdReturn := '';
  CurrentEditFile := '';
  CommandLine := '';

  if not Form.Options.Syntaxs.Enabled then Exit;

  EditBuffer := OTAGetCurrentEditBuffer;
  if Assigned(EditBuffer) then
  begin
    CurrentEditFile := EditBuffer.FileName;
    if {isProjectFile(CurrentEditFile) and} isPascalFile(CurrentEditFile) then
    begin
      DelDirectory(Form.Options.Syntaxs.TempDir);
      Mkdir(Form.Options.Syntaxs.TempDir);
      TempFile := IncludeTrailingPathDelimiter(Form.Options.Syntaxs.TempDir) + ExtractFileName(CurrentEditFile);

      if not Assigned(Strs) then Strs := TStringList.Create;
      Strs.Clear;
      Strs.Text := OTAGetEditString(EditBuffer);
      Strs.SaveToFile(TempFile);
      CommandLine := '"' + Form.Options.Syntaxs.CompileBin + '" -$O- -$W+ -H -Q -W ';
      CommandLine := CommandLine + _GetOutputSetting;
      CommandLine := CommandLine + _GetIncludes;
      CommandLine := CommandLine + _GetUnits;
      CommandLine := CommandLine + ' ' + Form.Options.Syntaxs.Addtionals;
      CommandLine := CommandLine + ' "' + TempFile + '"';
      //Debug('CommandLine', CommandLine);
    end;
  end;
  EditBuffer := nil;
end;

procedure TDGCompiler.OutputCompileInfos;
var
  Error: TDgError;
  TempStrs: TStringList;
  i, pos1, pos2: Integer;
  TempStr: string;
begin
  if not Form.Options.Syntaxs.Enabled then Exit;
  if Assigned(Form.EditControl) then
  begin
    Form.Errors := DGUtils.ClearObjectList(Form.Errors);
    TempStrs := TStringList.Create;
    try
      TempStrs.Text := cmdReturn;
      Debug('Command.Return', TempStrs);
      for i := 2 to TempStrs.Count - 1 do
      begin
        TempStr := TempStrs.Strings[i];
        if Length(Trim(TempStr)) = 0 then Continue;
        pos1 := Pos('(', TempStr);
        pos2 := Pos(')', TempStr);
        if (pos2 > pos1) and (pos1 > 0) then
        begin
          Error := TDgError.Create;
          Error.FileName := Copy(TempStr, 1, pos1 - 1);
          Error.LocalError := ExtractFilename(Error.FileName) = ExtractFileName(CurrentEditFile);
          Error.LineNumber := StrToIntDef(Copy(TempStr, pos1 + 1, pos2 - pos1 - 1), 0);
          Error.ErrorMsg := Trim(Copy(TempStr, pos2 + 1, Length(TempStr)));
          Error.IsError := Copy(Error.ErrorMsg, 1, 5) = 'Error';
          Error.IsWarning := Copy(Error.ErrorMsg, 1, 7) = 'Warning';
          Form.Errors.Add(Error);
        end;
      end;
    finally
      FreeandNil(TempStrs);
    end;
    Form.EditControl.Refresh;
  end;
end;

procedure TDGCompiler.CompileCurrentProject;
begin
  {
    dcc32 -$O- -$W+ -R -N. -LE. -LN. -E. -H -Q -W
    -I".\DGExperts"
    -LUdesignide
    -U".\DGExperts\bin";"C:\Program Files\Borland\Delphi7\Lib";"C:\Program Files\Borland\Delphi7\Bin"
    DGUtils.pas
  }
  try
    CmdReturn := ExecWithPipe(CommandLine, Form.Options.Syntaxs.TempDir);
  except
  end;
end;

end.

