unit DGUtils;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DGTypes, Contnrs, mwSimplePasPar, mwPasLexTypes, mwPasLex, ToolsAPI,
  ComCtrls;

{$I DGExperts.inc}
{.$DEFINE ENABLE_CSDEBUG}

function ExecWithPipe(CommandLine, Dir: string): string;

function DelDirectory(const Source: string): boolean;
function GetTempFolder: string;
function GetFolder(const Handle: HWND; const Title: string; var Folder: string): Boolean;

////////////////////////////////////////////////////////////////////////////////

procedure LoadDGExpertOptions(Options: TDgOptions);
procedure SaveDGExpertOptions(Options: TDgOptions);

////////////////////////////////////////////////////////////////////////////////

function AssignToken(PasLex: TmwPasLex): TPasToken;

////////////////////////////////////////////////////////////////////////////////

function OTAGetCurrentEditBuffer: IOTAEditBuffer;
function OTAGetCurrentEditView: IOTAEditView;

function OTAGetCurrentEditSelection: string;

procedure OTAEditInsertText(const Content: string);
procedure OTAEditDelete(const Count: Integer);
procedure OTAEditBackspaceDelete(const Count: Integer);
procedure OTAEditMoveRelative(const Row, Col: Integer);

function OTAGetCursorPos(var CursorLine: Integer; var CursorCol: Smallint): Boolean; overload;
function OTAGetCursorPos(var CursorPos: TOTAEditPos): Boolean; overload;
function OTAGetCursorPos: TOTAEditPos; overload;
procedure OTASetCursorPos(const CursorLine: Integer; const CursorCol: Smallint); overload;
procedure OTASetCursorPos(const CursorPos: TOTAEditPos); overload;

function OTAGetEditString(Editor: IOTASourceEditor): string;
procedure OTASetEditString(Editor: IOTASourceEditor; const Value: string);

function OTACodeInsight(var StrTemp: string): boolean;
function OTAParameterCodeInsight(var SelectIndex: integer): boolean;

////////////////////////////////////////////////////////////////////////////////

procedure Debug(); overload;
procedure Debug(Description: string); overload;
procedure Debug(Description: string; const Args: array of const); overload;
procedure Debug(Description: string; Content: string); overload;
procedure Debug(Description: string; Content: Boolean); overload;
procedure Debug(Description: string; Content: PChar); overload;
procedure Debug(Description: string; Content: Integer); overload;
procedure Debug(Description: string; Content: Extended); overload;
procedure Debug(Description: string; Content: DWord); overload;
procedure Debug(Description: string; Content: TStrings); overload;
procedure Debug(Description: string; Content: TObject); overload;
procedure Debug(Description: string; Content: Variant); overload;
procedure Debug(Description: string; Content: TBitmap); overload;

procedure DebugConfig(CColor: TColor);
procedure DebugAddSeparator;
procedure DebugEnterMethod(const MethodName: string; Color: DWORD = $AAFFFF);
procedure DebugExitMethod();

////////////////////////////////////////////////////////////////////////////////

procedure LoadBitmapFromRes(TheBitmap: string; Image: TBitmap);

////////////////////////////////////////////////////////////////////////////////

procedure EnableControls(Parent: TWinControl; const isEnable: Boolean);
function GetCurrentUser: string;
function GetDelphi7RootDir: string;
function GetDelphi7LibPath: string;
function GetDelphiStatusBar: TStatusBar;
function GetDelphiEditWindow: TCustomForm;
function GetDelphiEditControl: TCustomControl;
function isDelphiEditControl(const hWnd: HWND): Boolean;
function isPascalFile(const FileName: string): Boolean;
function isProjectFile(const FileName: string): Boolean;
function isNull(s: string): Boolean;
function isNotNull(s: string): Boolean;

function ClearObjectList(List: TObjectList): TObjectList;
function FreeObjectList(var List: TObjectList): TObjectList; // Always return nil

function isAllowTemplateRun(const LineText: string; CursorCol: Smallint): Boolean;
function SurroundCursorText(const LineText: string; const CursorCol: SmallInt): string;
function AheadCursorText(const LineText: string; const CursorCol: SmallInt): string;
procedure GetBracketMatch(EditView: IOTAEditView; EditBuffer: IOTAEditBuffer; EditControl: TCustomControl;
  var FTokenPos, FTokenMatchPos: TOTAEditPos; var TokenStr, TokenMatchStr: string);
procedure GetBracketMatch2(EditView: IOTAEditView; EditBuffer: IOTAEditBuffer; EditControl: TCustomControl;
  var FTokenPos, FTokenMatchPos: TOTAEditPos; var TokenStr, TokenMatchStr: string);

var
  SaveMethodName: string;

implementation

uses
{$IFDEF ENABLE_CSDEBUG}
  CSIntf,
{$ENDIF}
  StrUtils, TypInfo, Registry, ShellAPI, ShlObj;

function ExecWithPipe(CommandLine, Dir: string): string;
var
  HRead, HWrite: THandle;
  StartInfo: TStartupInfo;
  ProceInfo: TProcessInformation;
  b: Boolean;
  sa: TSecurityAttributes;
  inS: THandleStream;
  sRet: TStrings;
begin
  Result := '';
  FillChar(sa, sizeof(sa), 0);
  sa.nLength := sizeof(sa);
  sa.bInheritHandle := True;
  sa.lpSecurityDescriptor := nil;
  b := CreatePipe(HRead, HWrite, @sa, 0);
  if not b then raise Exception.Create(SysErrorMessage(GetLastError));

  FillChar(StartInfo, SizeOf(StartInfo), 0);
  StartInfo.cb := SizeOf(StartInfo);
  StartInfo.wShowWindow := SW_HIDE;
  StartInfo.dwFlags := STARTF_USESTDHANDLES + STARTF_USESHOWWINDOW;
  StartInfo.hStdError := HWrite;
  StartInfo.hStdInput := GetStdHandle(STD_INPUT_HANDLE);
  StartInfo.hStdOutput := HWrite;

  b := CreateProcess(nil, //lpApplicationName: PChar
    PChar(CommandLine), //lpCommandLine: PChar
    nil, //lpProcessAttributes: PSecurityAttributes
    nil, //lpThreadAttributes: PSecurityAttributes
    True, //bInheritHandles: BOOL
    CREATE_NEW_CONSOLE,
    PChar(Dir),
    nil,
    StartInfo,
    ProceInfo);

  if not b then raise Exception.Create(SysErrorMessage(GetLastError));
  WaitForSingleObject(ProceInfo.hProcess, INFINITE);

  inS := THandleStream.Create(HRead);
  if inS.Size > 0 then
  begin
    sRet := TStringList.Create;
    sRet.LoadFromStream(inS);
    Result := sRet.Text;
    sRet.Free;
  end;
  inS.Free;

  CloseHandle(HRead);
  CloseHandle(HWrite);
end;

function DelDirectory(const Source: string): boolean;
var
  fo: TSHFILEOPSTRUCT;
begin
  FillChar(fo, SizeOf(fo), 0);
  with fo do
  begin
    Wnd := 0;
    wFunc := FO_DELETE;
    pFrom := PChar(source + #0);
    pTo := #0#0;
    fFlags := FOF_NOCONFIRMATION + FOF_SILENT;
  end;
  Result := (SHFileOperation(fo) = 0);
end;

function GetTempFolder: string;
var
  pPath: PChar;
begin
  GetMem(pPath, MAX_PATH);
  GetTempPath(MAX_PATH, pPath);
  Result := IncludeTrailingPathDelimiter(pPath);
  FreeMem(pPath, MAX_PATH);
end;

function GetFolder(const Handle: HWND; const Title: string; var Folder: string): Boolean;
var
  BrowseInfo: TBrowseInfo;
  Id: PItemIDList;
  FN: array[0..MAX_PATH] of Char;
begin
  with BrowseInfo do
  begin
    hwndOwner := Handle;
    pidlRoot := nil;
    pszDisplayName := FN;
    lpszTitle := PChar(Title);
    ulFlags := 0;
    lpfn := nil;
  end;
  Id := SHBrowseForFolder(BrowseInfo);
  Result := Id <> nil;
  if Result then
  begin
    SHGetPathFromIDList(Id, FN);
    Folder := FN;
  end;
end;

procedure LoadDGExpertOptions(Options: TDgOptions);
var
  RegIni: TRegIniFile;
  DelphiDir: string;
begin
  RegIni := TRegIniFile.Create;
  DelphiDir := '"' + IncludeTrailingPathDelimiter(GetDelphi7RootDir);
  RegIni.RootKey := HKEY_CURRENT_USER;
  if not RegIni.OpenKey('SOFTWARE\DGExperts', True) then Exit;
  try
    with Options, RegIni do
    begin
      Templates.Enabled := ReadBool('Templates', 'Enabled', True);

      Gutters.Enabled := ReadBool('Gutters', 'Enabled', false);
      Gutters.AutoReInstall := ReadBool('Gutters', 'AutoReInstall', True);
      Gutters.LineNumber := ReadBool('Gutters', 'LineNumber', True);
      Gutters.CurrentLineImage := ReadBool('Gutters', 'CurrentLineImage', True);
      Gutters.LineNumberFontName := ReadString('Gutters', 'LineNumberFontName', 'Tahoma');
      Gutters.LineNumberFontSize := ReadInteger('Gutters', 'LineNumberFontSize', 8);
      Gutters.LineNumberFontColor := ReadInteger('Gutters', 'LineNumberFontColor', $808000);
      Gutters.LineNumberFontDefaultColor := ReadInteger('Gutters', 'LineNumberFontDefaultColor', clGray);
      Gutters.GutterWidth := ReadInteger('Gutters', 'GutterWidth', 30);

      Gutters.BookmarkImage := ReadBool('Gutters', 'BookmarkImage', True);
      Gutters.KeyImage := ReadBool('Gutters', 'KeyImage', True);
      Gutters.ErrorImage := ReadBool('Gutters', 'ErrorImage', True);
      Gutters.QuestionImage := ReadBool('Gutters', 'QuestionImage', True);
      Gutters.BookmarkBackground := ReadInteger('Gutters', 'BookmarkBackground', clYellow);
      Gutters.BookmarkForeground := ReadInteger('Gutters', 'BookmarkForeground', clRed);

      Matchs.Enabled := ReadBool('Matchs', 'Enabled', True);
      Matchs.EnableBlockMatch := ReadBool('Matchs', 'EnableBlockMatch', True);
      Matchs.EnableBlockMatchLineWidth := ReadInteger('Matchs', 'EnableBlockMatchLineWidth', 1);
      Matchs.BlockMatchLineStyle := ReadInteger('Matchs', 'BlockMatchStyle', 2);
      Matchs.EnableBracketMatch := ReadBool('Matchs', 'EnableBracketMatch', True);
      Matchs.BracketMatchBackground := ReadInteger('Matchs', 'BracketMatchBackground', clYellow);
      Matchs.BracketMatchForeground := ReadInteger('Matchs', 'BracketMatchForeground', clGray);
      Matchs.BracketMatchBold := ReadBool('Matchs', 'BracketMatchBold', True);

      Matchs.EnableBlockMatchDelay := ReadInteger('Matchs', 'BlockMatchDelay', 0);
      Matchs.BlockMatchBold := ReadBool('Matchs', 'BlockMatchBold', True);
      Matchs.BlockMatchUseHotkey := ReadBool('Matchs', 'BlockMatchUseHotkey', False);
      Matchs.BlockMatchCurrentLine := ReadBool('Matchs', 'BlockMatchCurrentLine', True);

      Matchs.BlockBackground1 := ReadInteger('Matchs', 'BlockBackground1', $FFFFFF);
      Matchs.BlockBackground2 := ReadInteger('Matchs', 'BlockBackground2', $FFAAFF);
      Matchs.BlockBackground3 := ReadInteger('Matchs', 'BlockBackground3', $FFFFFF);
      Matchs.BlockBackground4 := ReadInteger('Matchs', 'BlockBackground4', $FFFFFF);
      Matchs.BlockBackground5 := ReadInteger('Matchs', 'BlockBackground5', $FFFFFF);
      Matchs.BlockBackground6 := ReadInteger('Matchs', 'BlockBackground6', $FFFFFF);
      Matchs.BlockBackground7 := ReadInteger('Matchs', 'BlockBackground7', $AAFFFF);

      Matchs.BlockForeground1 := ReadInteger('Matchs', 'BlockForeground1', $000000);
      Matchs.BlockForeground2 := ReadInteger('Matchs', 'BlockForeground2', $408080);
      Matchs.BlockForeground3 := ReadInteger('Matchs', 'BlockForeground3', $004080);
      Matchs.BlockForeground4 := ReadInteger('Matchs', 'BlockForeground4', $00FF80);
      Matchs.BlockForeground5 := ReadInteger('Matchs', 'BlockForeground5', $804000);
      Matchs.BlockForeground6 := ReadInteger('Matchs', 'BlockForeground6', $C0C0C0);
      Matchs.BlockForeground7 := ReadInteger('Matchs', 'BlockForeground7', $FF0000);

      //    property CompileBin: string read FCompileBin write FCompileBin;
      //    property TempDir: string read FTempDir write FTempDir;
      //    property IncludesDir: string read FIncludesDir write FIncludesDir;
      //    property UnitsDir: string read FUnitsDir write FUnitsDir;
      //    property Addtionals: string read FAddtionals write FAddtionals;
      //    property Interval: Integer read FInterval write FInterval;
      Syntaxs.Enabled := ReadBool('Syntaxs', 'Enable', False);
      Syntaxs.CompileBin := ReadString('Syntaxs', 'CompileBin', 'dcc32.exe');
      Syntaxs.TempDir := ReadString('Syntaxs', 'TempDir', GetTempFolder + 'DGExperts');

      Syntaxs.IncludesDir := ReadString('Syntaxs', 'IncludesDir', '');

      if isNotNull(DelphiDir) then
        Syntaxs.UnitsDir := ReadString('Syntaxs', 'UnitsDir', DelphiDir + 'bin";' + DelphiDir + 'lib";' + DelphiDir + 'Projects\Bpl"')
      else
        Syntaxs.UnitsDir := ReadString('Syntaxs', 'UnitsDir', '');

      Syntaxs.Addtionals := ReadString('Syntaxs', 'Addtionals', '-LUdesignide');
      Syntaxs.Interval := ReadInteger('Syntaxs', 'Interval', 3000);
    end;
    RegIni.CloseKey;
  finally
    RegIni.Free;
  end;
  Debug('Load Options.Templates', Options.Templates);
  Debug('Load Options.Gutters', Options.Gutters);
  Debug('Load Options.Matchs', Options.Matchs);
  Debug('Load Options.Syntaxs', Options.Syntaxs);
  DebugAddSeparator;
end;

procedure SaveDGExpertOptions(Options: TDgOptions);
var
  RegIni: TRegIniFile;
begin
  Debug('Save Options.Templates', Options.Templates);
  Debug('Save Options.Gutters', Options.Gutters);
  Debug('Save Options.Matchs', Options.Matchs);
  Debug('Save Options.Syntaxs', Options.Syntaxs);
  DebugAddSeparator;

  RegIni := TRegIniFile.Create;
  RegIni.RootKey := HKEY_CURRENT_USER;
  if not RegIni.OpenKey('SOFTWARE\DGExperts', True) then Exit;
  try
    with Options, RegIni do
    begin
      WriteBool('Templates', 'Enabled', Templates.Enabled);

      WriteBool('Gutters', 'Enabled', Gutters.Enabled);
      WriteBool('Gutters', 'AutoReInstall', Gutters.AutoReInstall);
      WriteBool('Gutters', 'LineNumber', Gutters.LineNumber);
      WriteBool('Gutters', 'CurrentLineImage', Gutters.CurrentLineImage);
      WriteString('Gutters', 'LineNumberFont.Name', Gutters.LineNumberFontName);
      WriteInteger('Gutters', 'LineNumberFontSize', Gutters.LineNumberFontSize);
      WriteInteger('Gutters', 'LineNumberFontColor', Gutters.LineNumberFontColor);
      WriteInteger('Gutters', 'LineNumberFontDefaultColor', Gutters.LineNumberFontDefaultColor);
      WriteInteger('Gutters', 'GutterWidth', Gutters.GutterWidth);

      WriteBool('Gutters', 'BookmarkImage', Gutters.BookmarkImage);
      WriteBool('Gutters', 'KeyImage', Gutters.KeyImage);
      WriteBool('Gutters', 'ErrorImage', Gutters.ErrorImage);
      WriteBool('Gutters', 'QuestionImage', Gutters.QuestionImage);
      WriteInteger('Gutters', 'BookmarkBackground', Gutters.BookmarkBackground);
      WriteInteger('Gutters', 'BookmarkForeground', Gutters.BookmarkForeground);

      WriteBool('Matchs', 'Enabled', Matchs.Enabled);
      WriteBool('Matchs', 'EnableBlockMatch', Matchs.EnableBlockMatch);
      WriteBool('Matchs', 'EnableBracketMatch', Matchs.EnableBracketMatch);
      WriteInteger('Matchs', 'EnableBlockMatchLineWidth', Matchs.EnableBlockMatchLineWidth);
      WriteInteger('Matchs', 'BlockMatchStyle', Matchs.BlockMatchLineStyle);
      WriteInteger('Matchs', 'BracketMatchBackground', Matchs.BracketMatchBackground);
      WriteInteger('Matchs', 'BracketMatchForeground', Matchs.BracketMatchForeground);
      WriteBool('Matchs', 'BracketMatchBold', Matchs.BracketMatchBold);

      WriteInteger('Matchs', 'BlockMatchDelay', Matchs.EnableBlockMatchDelay);
      WriteBool('Matchs', 'BlockMatchBold', Matchs.BlockMatchBold);
      WriteBool('Matchs', 'BlockMatchItalic', Matchs.BlockMatchItalic);
      WriteBool('Matchs', 'BlockMatchUseHotkey', Matchs.BlockMatchUseHotkey);
      WriteBool('Matchs', 'BlockMatchCurrentLine', Matchs.BlockMatchCurrentLine);

      WriteInteger('Matchs', 'BlockBackground1', Matchs.BlockBackground1);
      WriteInteger('Matchs', 'BlockBackground2', Matchs.BlockBackground2);
      WriteInteger('Matchs', 'BlockBackground3', Matchs.BlockBackground3);
      WriteInteger('Matchs', 'BlockBackground4', Matchs.BlockBackground4);
      WriteInteger('Matchs', 'BlockBackground5', Matchs.BlockBackground5);
      WriteInteger('Matchs', 'BlockBackground6', Matchs.BlockBackground6);
      WriteInteger('Matchs', 'BlockBackground7', Matchs.BlockBackground7);

      WriteInteger('Matchs', 'BlockForeground1', Matchs.BlockForeground1);
      WriteInteger('Matchs', 'BlockForeground2', Matchs.BlockForeground2);
      WriteInteger('Matchs', 'BlockForeground3', Matchs.BlockForeground3);
      WriteInteger('Matchs', 'BlockForeground4', Matchs.BlockForeground4);
      WriteInteger('Matchs', 'BlockForeground5', Matchs.BlockForeground5);
      WriteInteger('Matchs', 'BlockForeground6', Matchs.BlockForeground6);
      WriteInteger('Matchs', 'BlockForeground7', Matchs.BlockForeground7);

      WriteBool('Syntaxs', 'Enable', Syntaxs.Enabled);
      WriteString('Syntaxs', 'CompileBin', Syntaxs.CompileBin);
      WriteString('Syntaxs', 'TempDir', Syntaxs.TempDir);
      WriteString('Syntaxs', 'IncludesDir', Syntaxs.IncludesDir);
      WriteString('Syntaxs', 'UnitsDir', Syntaxs.UnitsDir);
      WriteString('Syntaxs', 'Addtionals', Syntaxs.Addtionals);
      WriteInteger('Syntaxs', 'Interval', Syntaxs.Interval);
    end;
    RegIni.CloseKey;
  finally
    RegIni.Free;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

function AssignToken(PasLex: TmwPasLex): TPasToken;
begin
  Result := TPasToken.Create;
  Result.Token := PasLex.Token;
  Result.LineNumber := PasLex.LineNumber;
  Result.LinePos := PasLex.LinePos;
  Result.RunPos := PasLex.RunPos;
  Result.TokenPos := PasLex.TokenPos;
  Result.TokenLen := PasLex.TokenLen;
  Result.TokenID := PasLex.TokenID;
  Result.GenID := PasLex.GenID;
end;

////////////////////////////////////////////////////////////////////////////////

function OTAGetCurrentEditBuffer: IOTAEditBuffer;
var
  EServices: IOTAEditorServices;
begin
  try
    try
      EServices := (BorlandIDEServices as IOTAEditorServices);
      if Assigned(EServices) then Result := EServices.TopBuffer;
    finally
      EServices := nil;
    end;
  except
    Result := nil;
  end;
end;

function OTAGetCurrentEditView: IOTAEditView;
var
  EServices: IOTAEditorServices;
begin
  try
    try
      EServices := (BorlandIDEServices as IOTAEditorServices);
      if Assigned(EServices) then Result := EServices.TopView;
    finally
      EServices := nil;
    end;
  except
    Result := nil;
  end;
end;

function OTAGetCurrentEditSelection: string;
var
  StartPos, EndPos: Integer;
  FEditView: IOTAEditView;
  FEditBuffer: IOTAEditBuffer;
begin
  //  Get Selection Text in EditView
  Result := '';
  FEditView := OTAGetCurrentEditView;
  FEditBuffer := OTAGetCurrentEditBuffer;
  try
    if Assigned(FEditBuffer) and Assigned(FEditView) then
    begin
      StartPos := FEditView.CharPosToPos(FEditBuffer.BlockStart);
      EndPos := FEditView.CharPosToPos(FEditBuffer.BlockAfter);
      SetLength(Result, EndPos - StartPos - 1);
      FEditBuffer.CreateReader.GetText(StartPos, PChar(Result), Length(Result));
    end;
  finally
    FEditView := nil;
    FEditBuffer := nil;
  end;
end;

procedure OTAEditBackspaceDelete(const Count: Integer);
var
  EditView: IOTAEditView;
begin
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then
    begin
      EditView.Position.BackspaceDelete(Count);
      EditView.Paint;
    end;
  finally
    EditView := nil;
  end;
end;

procedure OTAEditDelete(const Count: Integer);
var
  EditView: IOTAEditView;
begin
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then
    begin
      EditView.Position.Delete(Count);
      EditView.Paint;
    end;
  finally
    EditView := nil;
  end;
end;

procedure OTAEditInsertText(const Content: string);
var
  EditView: IOTAEditView;
begin
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then
    begin
      EditView.Position.InsertText(Content);
      EditView.Paint;
    end;
  finally
    EditView := nil;
  end;
end;

procedure OTAEditMoveRelative(const Row, Col: Integer);
var
  EditView: IOTAEditView;
begin
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then
    begin
      EditView.Position.MoveRelative(Row, Col);
      EditView.Paint;
    end;
  finally
    EditView := nil;
  end;
end;

function OTAGetCursorPos(var CursorLine: Integer; var CursorCol: Smallint): Boolean;
var
  CursorPos: TOTAEditPos;
begin
  Result := OTAGetCursorPos(CursorPos);
  if Result then
  begin
    CursorLine := CursorPos.Line;
    CursorCol := CursorPos.Col;
  end;
end;

function OTAGetCursorPos(var CursorPos: TOTAEditPos): Boolean;
var
  EditView: IOTAEditView;
begin
  Result := False;
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then
    begin
      CursorPos := EditView.CursorPos;
      Result := True;
    end;
  finally
    EditView := nil;
  end;
end;

function OTAGetCursorPos: TOTAEditPos;
var
  EditView: IOTAEditView;
  CursorPos: TOTAEditPos;
begin
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then
    begin
      CursorPos := EditView.CursorPos;
      Result := CursorPos;
    end;
  finally
    EditView := nil;
  end;
end;

procedure OTASetCursorPos(const CursorPos: TOTAEditPos);
var
  EditView: IOTAEditView;
begin
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then
    begin
      EditView.SetCursorPos(CursorPos);
      EditView.Paint;
    end;
  finally
    EditView := nil;
  end;
end;

procedure OTASetCursorPos(const CursorLine: Integer; const CursorCol: Smallint); overload;
var
  CursorPos: TOTAEditPos;
begin
  CursorPos.Line := CursorLine;
  CursorPos.Col := CursorCol;
  OTASetCursorPos(CursorPos);
end;

function OTAGetEditString(Editor: IOTASourceEditor): string;
const
  BUF_SIZE = 8192;
var
  Reader: IOTAEditReader;
  StrStrm: TStringStream;
  Position: Integer;
  NrRead: Integer;
  Buf: array[0..BUF_SIZE] of Char;
begin
  Reader := Editor.CreateReader;
  StrStrm := TStringStream.Create('');
  try
    Position := 0;
    repeat
      NrRead := Reader.GetText(Position, @Buf, BUF_SIZE);
      Inc(Position, NrRead);
      if (NrRead < 0) or (NrRead > BUF_SIZE) then
      begin
        Result := '';
        Exit;
      end;
      Buf[NrRead] := #0;
      StrStrm.WriteString(PChar(@Buf));
    until NrRead < BUF_SIZE;
    Result := StrStrm.DataString;
  finally
    StrStrm.Free;
    Reader := nil;
  end;
end;

procedure OTASetEditString(Editor: IOTASourceEditor; const Value: string);
var
  Writer: IOTAEditWriter;
begin
  Writer := Editor.CreateUndoableWriter;
  try
    Writer.DeleteTo(MaxInt);
    Writer.Insert(PChar(Value));
  finally
    Writer := nil;
  end;
end;

function OTACodeInsight(var StrTemp: string): boolean;
var
  CIServices: IOTACodeInsightServices;
  CodeInsightManager: IOTACodeInsightManager;
begin
  Result := false;
  CIServices := (BorlandIDEServices as IOTACodeInsightServices);
  try
    if Assigned(CIServices) then
    begin
      CIServices.GetCurrentCodeInsightManager(CodeInsightManager);
      if Assigned(CodeInsightManager) then
        Result := CodeInsightManager.InvokeCodeCompletion(itManual, StrTemp);
    end;
  finally
    CodeInsightManager := nil;
    CIServices := nil;
  end;
end;

function OTAParameterCodeInsight(var SelectIndex: integer): boolean;
var
  CIServices: IOTACodeInsightServices;
  CodeInsightManager: IOTACodeInsightManager;
begin
  Result := false;
  CIServices := (BorlandIDEServices as IOTACodeInsightServices);
  try
    if Assigned(CIServices) then
    begin
      CIServices.GetCurrentCodeInsightManager(CodeInsightManager);
      if Assigned(CodeInsightManager) then
        Result := CodeInsightManager.InvokeParameterCodeInsight(itManual, SelectIndex);
    end;
  finally
    CodeInsightManager := nil;
    CIServices := nil;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure Debug(); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendException;
{$ENDIF }
end;

procedure Debug(Description: string); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendMsg(Description);
{$ENDIF }
end;

procedure Debug(Description: string; const Args: array of const); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendMsg(Format(Description, Args));
{$ENDIF }
end;

procedure Debug(Description, Content: string); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendString(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: Boolean); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendBoolean(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: PChar); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendPChar(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: Integer); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendInteger(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: Extended); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendFloat(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: DWord); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendNumAsHex(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: TStrings); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendStringList(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: TObject); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendObject(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: Variant); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendVariant(Description, Content);
{$ENDIF }
end;

procedure Debug(Description: string; Content: TBitmap); overload;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.SendBitmap(Description, Content);
{$ENDIF }
end;

procedure DebugConfig(CColor: TColor);
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.CategoryColor := CColor;
{$ENDIF }
end;

procedure DebugAddSeparator;
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.AddSeparator;
{$ENDIF }
end;

procedure DebugEnterMethod(const MethodName: string; Color: DWORD = $AAFFFF);
begin
{$IFDEF ENABLE_CSDEBUG }
  SaveMethodName := MethodName;
  CodeSite.EnterMethod(MethodName);
  CodeSite.CategoryColor := TColor(Color);
{$ENDIF }
end;

procedure DebugExitMethod();
begin
{$IFDEF ENABLE_CSDEBUG }
  CodeSite.CategoryColor := clWhite;
  CodeSite.ExitMethod(SaveMethodName);
  CodeSite.AddSeparator;
{$ENDIF }
end;

////////////////////////////////////////////////////////////////////////////////

procedure LoadBitmapFromRes(TheBitmap: string; Image: TBitmap);
var
  ResHandle: THandle;
  MemHandle: THandle;
  MemStream: TMemoryStream;
  ResPtr: PByte;
  ResSize: Longint;
begin
  ResHandle := FindResource(hInstance, PChar(TheBitmap), 'BMP');
  MemHandle := LoadResource(hInstance, ResHandle);
  ResPtr := LockResource(MemHandle);
  MemStream := TMemoryStream.Create;
  try
    ResSize := SizeOfResource(hInstance, ResHandle);
    MemStream.SetSize(ResSize);
    MemStream.Write(ResPtr^, ResSize);
    FreeResource(MemHandle);
    MemStream.Seek(0, 0);
    Image.LoadFromStream(MemStream);
  finally
    MemStream.Free;
  end;
end;

procedure EnableControls(Parent: TWinControl; const isEnable: Boolean);
var
  i, Count: Integer;
  ctl: TControl;
begin
  Count := Parent.ControlCount - 1;
  for i := 0 to Count do
  begin
    ctl := Parent.Controls[i];
    if (ctl is TWinControl) then
    begin
      if TWinControl(ctl).ControlCount > 0 then
        EnableControls(TWinControl(ctl), isEnable);
      ctl.Enabled := isEnable;
    end;
  end;
end;

function GetCurrentUser: string;
var
  Size: DWord;
begin
  Size := 255;
  Setlength(Result, 255);
  GetUsername(PChar(Result), Size);
  Setlength(Result, Size);
end;

function GetDelphi7RootDir: string;
begin
  Result := IncludeTrailingPathDelimiter(GetEnvironmentVariable('Delphi'));
end;

function GetDelphi7LibPath: string;
var
  RegIni: TRegIniFile;
begin
  //  HKEY_CURRENT_USER\Software\Borland\Delphi\7.0\Library
  //  "Search Path"
  RegIni := TRegIniFile.Create;
  RegIni.RootKey := HKEY_CURRENT_USER;
  if not RegIni.OpenKey('Software\Borland\Delphi\7.0', False) then Exit;
  try
    Result := RegIni.ReadString('Library', 'Search Path', '');
    Result := AnsiReplaceText(Result, '$(Delphi)\', GetDelphi7RootDir);
    Result := '"' + Result + '"';
    Result := AnsiReplaceText(Result, ';', '";"');  
    RegIni.CloseKey;
  finally
    RegIni.Free;
  end;     
end;   

function GetDelphiStatusBar: TStatusBar;
var
  EditView: IOTAEditView;
begin
  Result := nil;
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then Result := EditView.GetEditWindow.StatusBar;
  finally
    EditView := nil;
  end;
end;

function GetDelphiEditWindow: TCustomForm;
var
  //  i: Integer;
  EditView: IOTAEditView;
begin
  Result := nil;
  EditView := OTAGetCurrentEditView;
  try
    if Assigned(EditView) then Result := EditView.GetEditWindow.Form;
  finally
    EditView := nil;
  end;
end;

function GetDelphiEditControl: TCustomControl;
var
  EditWnd: TCustomForm;

  procedure EnumAllChild(wc: TWinControl);
  var
    TempControl: TWinControl;
    j, Count: Integer;
  begin
    Count := wc.ControlCount - 1;
    for j := 0 to Count do
    begin
      if (wc.Controls[j] is TWinControl) then
      begin
        TempControl := TWinControl(wc.Controls[j]);
        if TempControl.ClassName = 'TEditControl' then
        begin
          Result := TCustomControl(TempControl);
          Debug('Assign Delphi TEditControl', Result);
          Exit;
        end
        else if TempControl.ControlCount > 0 then
          EnumAllChild(TempControl);
      end;
    end;
  end;

begin
  DebugEnterMethod('Find.Delphi.Control');
  try
    Result := nil;
    EditWnd := GetDelphiEditWindow;
    Debug('Assign Delphi TEditWindow', EditWnd);
    try
      if Assigned(EditWnd) then EnumAllChild(EditWnd);
    except
    end;
  finally
    DebugExitMethod();
  end;
end;

function isDelphiEditControl(const hWnd: HWND): Boolean;
var
  className: string;
begin
  SetLength(className, 128);
  SetLength(className, GetClassName(hWnd, pchar(className), 128));
  Result := className = 'TEditControl';
end;

function isPascalFile(const FileName: string): Boolean;
var
  Ext: string;
begin
  Ext := UpperCase(ExtractFileExt(FileName));
  Result := (Ext = '.PAS') or (Ext = '.DPR');
end;

function isProjectFile(const FileName: string): Boolean;
var
  i: Integer;
  Project: IOTAProject;
  ModuleInfo: IOTAModuleInfo;
begin
  Project := ToolsAPI.GetActiveProject;
  Result := False;
  try
    if Assigned(Project) then
    begin
      for i := 0 to Project.GetModuleCount - 1 do
      begin
        ModuleInfo := Project.GetModule(i);
        // Debug('ModuleInfo.FileName', ModuleInfo.FileName);
        if UpperCase(ModuleInfo.FileName) = UpperCase(FileName) then
        begin
          Result := True;
          Break;
        end;
      end;
    end;
  finally
    ModuleInfo := nil;
    Project := nil;
  end;
end;

function isNull(s: string): Boolean;
begin
  Result := Length(Trim(s)) = 0;
end;

function isNotNull(s: string): Boolean;
begin
  Result := not isNull(s);
end;

function ClearObjectList(List: TObjectList): TObjectList;
begin
  FreeObjectList(List);
  Result := TObjectList.Create(false);
end;

function FreeObjectList(var List: TObjectList): TObjectList;
var
  i, Count: Integer;
begin
  Result := nil;
  if (Assigned(List)) then
  begin
    Count := List.Count - 1;
    for i := 0 to Count do
      List.Items[i].Free;
    List.Clear;
    FreeAndNil(List);
  end;
end;

function SurroundCursorText(const LineText: string; const CursorCol: SmallInt): string;
var
  i, EndPos, StartPos, StrLength: Integer;
  Str: string;

  function isValidIdentifier(s: string): Boolean;
  var
    tmp: Integer;
  begin
    Result := False;
    tmp := Ord(s[1]);
    if ((tmp >= 65) and (tmp <= 90)) or // in ['A'..'Z']
    ((tmp >= 97) and (tmp <= 122)) then // in ['a'..'z']
      Result := True;
  end;

begin
  Result := '';
  EndPos := CursorCol;
  StartPos := 1;
  Str := LineText + '   ';
  if CursorCol > Length(Str) then exit;

  for i := CursorCol - 1 downto 0 do
  begin
    StartPos := i;
    if (i > 0) and (not isValidIdentifier(Str[i])) then break;
  end;

  if isValidIdentifier(Str[CursorCol]) then
  begin
    StrLength := Length(str);
    for i := CursorCol + 1 to StrLength do
    begin
      EndPos := i;
      if not isValidIdentifier(str[i]) then break;
    end;
  end;

  if EndPos - StartPos - 1 > 0 then
    Result := Copy(Str, StartPos + 1, EndPos - StartPos - 1);
end;

// ???????????????????????????
// ?????????????????????????????????????

procedure GetBracketMatch2(EditView: IOTAEditView; EditBuffer: IOTAEditBuffer; EditControl: TCustomControl;
  var FTokenPos, FTokenMatchPos: TOTAEditPos; var TokenStr, TokenMatchStr: string);
var
  LineText: string;

  function _FindMatchTokenDown(FindToken, MatchToken: string): TOTAEditPos;
  var
    i, Layer: Integer;
  begin
    Result.Col := 0;
    Result.Line := 0;
    Layer := 1;
    for i := FTokenPos.Col + 1 to Length(LineText) do
    begin
      if LineText[i] = FindToken then Inc(Layer);
      if LineText[i] = MatchToken then
      begin
        Dec(Layer);
        if Layer = 0 then
        begin
          Result.Line := FTokenPos.Line;
          Result.Col := i;
          Exit;
        end;
      end;
    end;
  end;

  function _FindMatchTokenUp(FindToken, MatchToken: string): TOTAEditPos;
  var
    i, Layer: Integer;
  begin
    Result.Col := 0;
    Result.Line := 0;
    Layer := 1;
    for i := FTokenPos.Col - 1 downto 1 do
    begin
      if LineText[i] = FindToken then Inc(Layer);
      if LineText[i] = MatchToken then
      begin
        Dec(Layer);
        if Layer = 0 then
        begin
          Result.Line := FTokenPos.Line;
          Result.Col := i;
          Exit;
        end;
      end;
    end;
  end;

  function _FindMatchToken(FindToken, MatchToken: string; Down: Boolean): TOTAEditPos;
  begin
    if Down then
      Result := _FindMatchTokenDown(FindToken, MatchToken)
    else // if not Down, means Up.
      Result := _FindMatchTokenUp(FindToken, MatchToken);
  end;
begin
  FTokenMatchPos.Col := 0;
  FTokenMatchPos.Line := 0;

  if OTAGetCursorPos(FTokenPos) then
  begin
    LineText := TypInfo.GetStrProp(EditControl, 'LineText');
    TokenStr := LineText[FTokenPos.Col];
    TokenMatchStr := '';
    if TokenStr = '(' then
    begin
      TokenMatchStr := ')';
      FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, True)
    end
    else if TokenStr = ')' then
    begin
      TokenMatchStr := '(';
      FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, False)
    end
    else if TokenStr = '[' then
    begin
      TokenMatchStr := ']';
      FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, True)
    end
    else if TokenStr = ']' then
    begin
      TokenMatchStr := '[';
      FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, False)
    end
    else if TokenStr = '{' then
    begin
      TokenMatchStr := '}';
      FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, True)
    end
    else if TokenStr = '}' then
    begin
      TokenMatchStr := '{';
      FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, False)
    end
  end;

end;

procedure GetBracketMatch(EditView: IOTAEditView; EditBuffer: IOTAEditBuffer; EditControl: TCustomControl;
  var FTokenPos, FTokenMatchPos: TOTAEditPos; var TokenStr, TokenMatchStr: string);
var
  LineText: string;
  Srcs: TStringList;

  function _FindMatchTokenDown(FindToken, MatchToken: string): TOTAEditPos;
  var
    i, j, Layer: Integer;
  begin
    Result.Col := 0;
    Result.Line := 0;
    Layer := 1;
    LineText := Srcs.Strings[FTokenPos.Line - 1];
    for i := FTokenPos.Col + 1 to Length(LineText) do
    begin
      if LineText[i] = FindToken then Inc(Layer);
      if LineText[i] = MatchToken then
      begin
        Dec(Layer);
        if Layer = 0 then
        begin
          Result.Line := FTokenPos.Line;
          Result.Col := i;
          Exit;
        end;
      end;
    end;

    for i := FTokenPos.Line to Srcs.Count - 1 do
    begin
      LineText := Srcs.Strings[i];
      for j := 1 to Length(LineText) do
      begin
        if LineText[j] = FindToken then Inc(Layer);
        if LineText[j] = MatchToken then
        begin
          Dec(Layer);
          if Layer = 0 then
          begin
            Result.Line := i + 1;
            Result.Col := j;
            Exit;
          end;
        end;
      end;
    end;
  end;

  function _FindMatchTokenUp(FindToken, MatchToken: string): TOTAEditPos;
  var
    i, j, Layer: Integer;
  begin
    Result.Col := 0;
    Result.Line := 0;
    Layer := 1;
    LineText := Srcs.Strings[FTokenPos.Line - 1];
    for i := FTokenPos.Col - 1 downto 1 do
    begin
      if LineText[i] = FindToken then Inc(Layer);
      if LineText[i] = MatchToken then
      begin
        Dec(Layer);
        if Layer = 0 then
        begin
          Result.Line := FTokenPos.Line;
          Result.Col := i;
          Exit;
        end;
      end;
    end;

    for i := FTokenPos.Line - 2 downto 0 do
    begin
      LineText := Srcs.Strings[i];
      for j := Length(LineText) downto 1 do
      begin
        if LineText[j] = FindToken then Inc(Layer);
        if LineText[j] = MatchToken then
        begin
          Dec(Layer);
          if Layer = 0 then
          begin
            Result.Line := i + 1;
            Result.Col := j;
            Exit;
          end;
        end;
      end;
    end;
  end;

  function _FindMatchToken(FindToken, MatchToken: string; Down: Boolean): TOTAEditPos;
  begin
    Srcs.Text := OTAGetEditString(EditBuffer);
    if Down then
      Result := _FindMatchTokenDown(FindToken, MatchToken)
    else // if not Down, means Up.
      Result := _FindMatchTokenUp(FindToken, MatchToken);
  end;

begin
  Assert(Assigned(EditView), 'EditView is nil.');
  Assert(Assigned(EditBuffer), 'EditBuffer is nil.');

  FTokenMatchPos.Col := 0;
  FTokenMatchPos.Line := 0;
  Srcs := TStringList.Create;
  try
    if OTAGetCursorPos(FTokenPos) then
    begin
      LineText := TypInfo.GetStrProp(EditControl, 'LineText');
      TokenStr := LineText[FTokenPos.Col];
      TokenMatchStr := '';
      if TokenStr = '(' then
      begin
        TokenMatchStr := ')';
        FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, True)
      end
      else if TokenStr = ')' then
      begin
        TokenMatchStr := '(';
        FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, False)
      end
      else if TokenStr = '[' then
      begin
        TokenMatchStr := ']';
        FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, True)
      end
      else if TokenStr = ']' then
      begin
        TokenMatchStr := '[';
        FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, False)
      end
      else if TokenStr = '{' then
      begin
        TokenMatchStr := '}';
        FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, True)
      end
      else if TokenStr = '}' then
      begin
        TokenMatchStr := '{';
        FTokenMatchPos := _FindMatchToken(TokenStr, TokenMatchStr, False)
      end
    end;
  finally
    FreeAndNil(Srcs);
  end;
end;

function AheadCursorText(const LineText: string; const CursorCol: SmallInt): string;
var
  i, StartPos: Integer;
  Str: string;

  function isValidTemplate(s: string): Boolean;
  var
    tmp: Integer;
  begin
    Result := False;
    tmp := Ord(s[1]);
    if ((tmp >= 33) and (tmp <= 126)) then // Visual ASCII
      Result := True;
  end;

begin
  Result := '';
  StartPos := 1;
  Str := LineText + '   ';
  if CursorCol > Length(Str) then exit;

  for i := CursorCol - 2 downto 0 do
  begin
    StartPos := i;
    if (i > 0) and (not isValidTemplate(Str[i])) then break;
  end;

  if CursorCol - StartPos - 1 > 0 then
    Result := Trim(Copy(Str, StartPos + 1, CursorCol - StartPos - 1))
end;

function isAllowTemplateRun(const LineText: string; CursorCol: Smallint): Boolean;
//  just process One-Line, need improve.
//  but if process whole source, maybe lost some time.
var
  LineTextType: string;

  function _ParseLineTextType(const LineText: string): string;
  var
    i, Size: Integer;
    str1, str2: string;
    CommentFlag1, // {   } Comment
    CommentFlag2, // (* *) Comment
    CommentFlag3, //  //   Coment
    StringFlag //   '~~~' String
    : Boolean;
  begin
    CommentFlag1 := False;
    CommentFlag2 := False;
    CommentFlag3 := False;
    StringFlag := False;
    Size := Length(LineText);
    i := 0;

    while i < Size do
    begin
      i := i + 1;
      str1 := LineText[i];
      str2 := ' ';
      if i < Size then str2 := LineText[i + 1];

      if (str1 = '{') then CommentFlag1 := True;
      if (str1 = '(') and (str2 = '*') then CommentFlag2 := True;
      if (str1 = '/') and (str2 = '/') then CommentFlag3 := True;

      if (str1 = '*') and (str2 = ')') and CommentFlag2 then
      begin
        CommentFlag2 := False;
        Result := Result + 'CC';
        i := i + 1;
        Continue;
      end;

      if (str1 = '}') and CommentFlag1 then
      begin
        CommentFlag1 := False;
        Result := Result + 'C';
        Continue;
      end;

      if (not CommentFlag1) and (not CommentFlag2) and (not CommentFlag3) then
      begin
        if (str1 = '''') then
          StringFlag := not StringFlag;
      end;

      //  Constract Result
      if (CommentFlag1 or CommentFlag2 or CommentFlag3) then
        Result := Result + 'C' // Comment
      else if StringFlag then
        Result := Result + 'S' // String
      else
        Result := Result + 'N'; // Normal, can do Template Action
    end;
  end;

begin
  LineTextType := _ParseLineTextType(LineText);
  Result := LineTextType[CursorCol - 1] = 'N';
end;

end.

